/*
 * Copyright (c) 1999 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache 
 *    Project. <http://java.apache.org/>"
 *
 * 4. The names "Java Apache Element Construction Set", "Java Apache ECS" and 
 *    "Java Apache Project" must not be used to endorse or promote products 
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called 
 *    "Java Apache Element Construction Set" nor "Java Apache ECS" appear 
 *    in their names without prior written permission of the 
 *    Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache 
 *    Project. <http://java.apache.org/>"
 *    
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 *
 */
package org.apache.ecs;

import java.io.OutputStream;
import java.io.PrintWriter;

/**
    This class creates a Document container, for convience.

    @version $Id: Document.java,v 1.3 1999/05/27 03:52:24 stephan Exp $
    @author <a href="mailto:snagy@servletapi.com">Stephan Nagy</a>
    @author <a href="mailto:jon@clearink.com">Jon S. Stevens</a>
*/
public class Document
{
    private Html html; // this is the actual container for head and body
    private Head head;
    private Body body;
    private Title title;
    private String codeset = null;
    
    {
        html = new Html();
        head = new Head();
        title = new Title();
        body = new Body();
        
        head.addElement(title);
        html.addElement(head);
        html.addElement(body);        
    }
    
    /**
        Basic constructor.
    */
    public Document()
    {
    }

    /**
        Basic constructor. Sets the codeset for the page output.
    */
    public Document(String codeset)
    {
        setCodeset(codeset);
    }

    /**
        Get the head element for this document container.
    */
    public Head getHead()
    {
        return(head);
    }
    
    /**
        Set the head element for this document container.
    */
    public Document setHead(Head set_head)
    {
        this.head = set_head;
        return(this);
    }
    
    /**
        Append to the head element for this document container.
        @param value adds to the value between the head tags
    */
    public Document appendHead(Element value)
    {
        head.addElement(value);
        return(this);
    }

    /**
        Append to the head element for this document container.
        @param value adds to the value between the head tags
    */
    public Document appendHead(String value)
    {
        head.addElement(value);
        return(this);
    }

    /**
        Get the body element for this document container.
    */
    public Body getBody()
    {
        return(body);
    }

    /**
        Set the Body element for this document container.
    */
    public Document setBody(Body set_body)
    {
        this.body = set_body;
        return(this);
    }
    
    /**
        Append to the body element for this document container.
        @param value adds to the value between the body tags
    */
    public Document appendBody(Element value)
    {
        body.addElement(value);
        return(this);
    }

    /**
        Append to the body element for this document container.
        @param value adds to the value between the body tags
    */
    public Document appendBody(String value)
    {
        body.addElement(value);
        return(this);
    }

    /**
        Get the title element for this document container.
    */
    public Title getTitle()
    {
        return(title);
    }

    /**
        Set the Title element for this document container.
    */
    public Document setTitle(Title set_title)
    {
        this.title = set_title;
        return(this);
    }
    
    /**
        Append to the title element for this document container.
        @param value adds to the value between the title tags
    */
    public Document appendTitle(Element value)
    {
        title.addElement(value);
        return(this);
    }

    /**
        Append to the title element for this document container.
        @param value adds to the value between the title tags
    */
    public Document appendTitle(String value)
    {
        title.addElement(value);
        return(this);
    }

    /**
     * Sets the codeset for this Document
     */
    public void setCodeset ( String codeset )
    {
        this.codeset = codeset;
    }

    /**
     * Gets the codeset for this Document
     *
     * @return    the codeset 
     */
    public String getCodeset()
    {
        return this.codeset;
    }
    
    /**
        Write the container to the OutputStream
    */
    public void output(OutputStream out)
    {
        // Document is just a convient wrapper for Html call Html.output
        html.output(out);
    }

    /**
        Write the container to the PrinteWriter
    */
    public void output(PrintWriter out)
    {
        // Document is just a convient wrapper for Html call Html.output
        html.output(out);
    }

    /**
        Override the toString() method so that it prints something meaningful.
    */
    public final String toString()
    {
        if ( getCodeset() != null )
            return (html.toString(getCodeset()));
        else
            return(html.toString());
    }

    /**
        Override the toString() method so that it prints something meaningful.
    */
    public final String toString(String codeset)
    {
        return(html.toString(codeset));
    }
}