/*
 * Copyright (c) 1999 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache 
 *    Project. <http://java.apache.org/>"
 *
 * 4. The names "Java Apache Element Construction Set", "Java Apache ECS" and 
 *    "Java Apache Project" must not be used to endorse or promote products 
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called 
 *    "Java Apache Element Construction Set" nor "Java Apache ECS" appear 
 *    in their names without prior written permission of the 
 *    Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache 
 *    Project. <http://java.apache.org/>"
 *    
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 *
 */
package org.apache.ecs;

import java.util.Hashtable;
import java.io.PrintWriter;
import java.io.OutputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ResourceBundle;

/**
    This class implements the ElementFactory.

    @version $Id: Element.java,v 1.12 1999/06/09 00:31:59 jonbolt Exp $
    @author <a href="mailto:snagy@servletapi.com">Stephan Nagy</a>
    @author <a href="mailto:jon@clearink.com">Jon S. Stevens</a>
*/
class Element implements ElementFactory
{
    protected static ResourceBundle resource;

    // private initializer.
    {
        try
        {   
            resource = ResourceBundle.getBundle("org.apache.ecs.ecs");
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
    }

    /**
        Filter to use to escape input
    */
    private static Filter filter = new CharacterFilter();

    /**
        position of tag relative to start and end.
    */
    private int position = Integer.parseInt(resource.getString("position"));
    
    /**
        Should we filter the value of &lt;&gt;VALUE&lt;/&gt;
    */
    private boolean filter_state = new Boolean(resource.getString("filter_state")).booleanValue();

    /**
        Version number for codebase
    */
    private static final String VERSION = "1.0b2";

    /**
        Default value to set case type
    */
    private int case_type = Integer.parseInt(resource.getString("case_type"));

    /**
        Where to get the element definition from
    */
    private Hashtable element = new Hashtable();

    /**
        Does this element need a closing tag?
    */
    private boolean end_element = new Boolean(resource.getString("end_element")).booleanValue();

    /**
        What codeset are we going to use the default is 8859_1
    */
    private String codeset = resource.getString("codeset");

    /**
        What the start modifier should be
    */
    private char start_modifier = ' ';

    /**
        What the end modifier should be
    */
    private char end_modifier = ' ';

    /**
        What the modifier should be...optimization variable
    */
    protected boolean start_modifier_defined = false;

    /**
        What the modifier should be...optimization variable
    */
    protected boolean end_modifier_defined = false;

    private char start_tag = resource.getString("start_tag").charAt(0);
    private char end_tag = resource.getString("end_tag").charAt(0);
    private String __text = new String();

    /**
        Location to store element_type at.
    */
    private String element_type = new String();

    /**
        Base class for defining all elements.
    */
    protected Element()
    {
    }

    /**
        Get the Hashtable representation of an element.
    */
    protected Hashtable getElementHashEntry()
    {
        return(element);
    }

    /**
        set the filter state of the element.
        @param filter_state (true|false) do we want to use a filter? default true;
    */
    public Element setFilterState(boolean filter_state)
    {
        this.filter_state = filter_state;
        return(this);
    }

    /**
        get the filter state of the element.
    */
    public boolean getFilterState()
    {
        return(filter_state);
    }

    /**
        Get the filter that is currently in use.
    */
    public Filter getFilter()
    {
        return(filter);
    }

    /**
        Override the default filter with a user supplied filter.
    */
    public void setFilter(Filter filter)
    {
        this.filter = filter;
    }

    /**
        What version of htmlGen is this?
    */
    public String getVersion()
    {
        return(VERSION);
    }

    /**
        Determine what case the html element should be rendered. Default is MIXEDCASE
    */
    public void setCase(int case_type)
    {
        this.case_type = case_type;
    }

    /**
        Find out what case the element is supposed to be rendered in.
    */
    public int getCase()
    {
        return(case_type);
    }

    /**
        Determine what type of element this is A,BR,EM,ACRONYM
    */
    public void setElementType(String element_type)
    {
        this.element_type = element_type;
    }

    /**
        Find out what type of element this is.
    */
    public String getElementType()
    {
        return(element_type);
    }

    /**
        Determine what codeset should be used.
    */
    public void setCodeSet(String codeset)
    {
        this.codeset = codeset;
    }

    /**
        Find out what the current codeset is.
    */
    public String getCodeSet()
    {
        return(codeset);
    }

    /**
        Does this element need a closing tag. Default is true
    */
    public void setNeedClosingTag(boolean close)
    {
        end_element = close;
    }

    /**
        Determine if this element needs to be closed.
    */
    public boolean getNeedClosingTag()
    {
        return(end_element);
    }

    /**
        Set the position of the tag to be rendered at in relation to start_element_char
    */
    public void setTagPosition(int position)
    {
        this.position = position;
    }

    /**
        Get the position the tag is to be rendered at.
    */
    public int getTagPosition()
    {
        return(position);
    }

    /**
        Alter the case of this element so that it is rendered according
        to whatever the value of case_type is.  Default is MIXEDCASE
    */
    protected String alterCase(String value)
    {
        switch (getCase())
        {
            case UPPERCASE:
                return value.toUpperCase();
            case LOWERCASE:
                return value.toLowerCase();
            default:
                return value;
        }
    }

    /**
        Set the element modifier the default is no modifier.
        @param modifier the modifier for the element a ? would result <br>
        in &lt;? ?&gt;
    */
    public Element setModifier(char modifier)
    {
        setStartModifier(modifier);
        setEndModifier(modifier);
        setStartModifierDefined(true);
        setEndModifierDefined(true);
        return(this);
    }

    /**
        Set the element modifier the default is no modifier.
        @param modifier the modifier for the element a ? would result <br>
        in &lt;? &gt;
    */
    public Element setStartModifier(char modifier)
    {
        this.start_modifier = modifier;
        setStartModifierDefined(true);
        return(this);
    }

    /**
        Set the element modifier the default is no modifier.
        @param modifier the modifier for the element a ? would result <br>
        in &lt; ?&gt;
    */
    public Element setEndModifier(char modifier)
    {
        this.end_modifier = modifier;
        setEndModifierDefined(true);
        return(this);
    }

    /**
        Set the element modifier the default is no modifier.
        @param modifier the modifier for the element a ? would result <br>
        in &lt;? ?&gt;
    */
    public Element setModifier(char start_modifier,char end_modifier)
    {
        this.start_modifier = start_modifier;
        this.end_modifier = end_modifier;
        setStartModifierDefined(true);
        setEndModifierDefined(true);
        return(this);
    }
    
    /**
        What is the element modifier for this tag.
    */
    public char getStartModifier()
    {
        return(start_modifier);
    }

    /**
        What is the element modifier for this tag.
    */
    public char getEndModifier()
    {
        return(end_modifier);
    }

    /**
        Remove the element modifier for this tag.
    */
    public Element removeStartModifier()
    {
        setStartModifierDefined(false);
        return(this);
    }

    /**
        Remove the element modifier for this tag.
    */
    public Element removeEndModifier()
    {
        setEndModifierDefined(false);
        return(this);
    }

    /**
        Modifier optimization
    */
    private void setStartModifierDefined(boolean val)
    {
        this.start_modifier_defined = val;
    }

    /**
        Modifier optimization
    */
    private void setEndModifierDefined(boolean val)
    {
        this.end_modifier_defined = val;
    }

    /**
        Modifier optimization
    */
    protected boolean getStartModifierDefined()
    {
        return this.start_modifier_defined;
    }

    /**
        Modifier optimization
    */
    protected boolean getEndModifierDefined()
    {
        return this.end_modifier_defined;
    }

    /**
        Set the start tag character.
    */
    public void setStartTagChar(char start_tag)
    {
        this.start_tag = start_tag;
    }

    /**
        Get the start tag character.
    */
    public char getStartTagChar()
    {
        return(start_tag);
    }

    /**
        Set the end tag character.
    */
    public void setEndTagChar(char end_tag)
    {
        this.end_tag = end_tag;
    }

    /**
        Get the end tag character.
    */
    public char getEndTagChar()
    {
        return(end_tag);
    }

    public String getTagText()
    {
        return(__text);
    }

    public void setTagText(String text)
    {
        this.__text = (text);
    }

    protected String createStartTag()
    {
        String out = null;
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        BufferedOutputStream bos = new BufferedOutputStream(baos);
        try
        {
            bos.write(getStartTagChar());

            if(start_modifier_defined)
            {
                bos.write(getStartModifier());
            }
            bos.write(getElementType().getBytes());

            if(end_modifier_defined)
            {
                bos.write(getEndModifier());
            }
            bos.write(getEndTagChar());
            bos.flush();
            out = baos.toString();
            bos.close();
            baos.close();
        }
        catch(IOException ioe)
        {
            ioe.printStackTrace();
        }
        return(out);
    }

    protected String createEndTag()
    {
        String out = null;
        setStartModifier('/');

        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        BufferedOutputStream bos = new BufferedOutputStream(baos);
        try
        {
            bos.write(getStartTagChar());

            if(start_modifier_defined)
            {
                bos.write(getStartModifier());
            }
            bos.write(getElementType().getBytes());

            if(end_modifier_defined)
            {
                bos.write(getEndModifier());
            }
            bos.write(getEndTagChar());
            bos.flush();
            out = baos.toString();
            bos.close();
            baos.close();
        }
        catch(IOException ioe)
        {
            ioe.printStackTrace();
        }
        return(out);
    }

    /**
        Override toString() method to print something meaningful
    */
    public final String toString()
    {
       String out = null;
       try
        {
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            BufferedOutputStream bos = new BufferedOutputStream(baos);
            output(bos);
            bos.flush();
            out = baos.toString();
            bos.close();
            baos.close();
        }
        catch (IOException ioe)
        {
        }
        return(out);
    }

    /**
        Override toString(encode) method to print something meaningful
    */
    public final String toString(String codeset)
    {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        BufferedOutputStream bos = new BufferedOutputStream(baos);
        String out = null;
        try
        {
            output(bos);
            bos.flush();
            out = baos.toString(codeset);
            bos.close();
            baos.close();
        }
        catch (UnsupportedEncodingException use)
        {
        }
        catch (IOException ioe)
        {
        }
        return(out);
    }

    /**
        Add the element to the designated OutputStream.
    */
    public void output(OutputStream out)
    {
        try
        {
            out.write(createStartTag().getBytes());

            if(getFilterState())
                out.write(getFilter().process(getTagText().toString()).getBytes());
            else
                out.write(getTagText().toString().getBytes());

            if (getNeedClosingTag())
                out.write(createEndTag().getBytes());
        }
        catch (IOException ioe)
        {
            ioe.printStackTrace(new PrintWriter(out));
        }
    }

    /**
        Add element to the designated PrintWriter.
    */
    public void output(PrintWriter out)
    {
        out.write(createStartTag());
        if(getFilterState())
            out.write(getFilter().process(getTagText().toString()));
        else
            out.write(getTagText().toString());

        if (getNeedClosingTag())
            out.write(createEndTag());
    }
}