/*
 * Copyright (c) 1999 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache 
 *    Project. <http://java.apache.org/>"
 *
 * 4. The names "Java Apache Element Construction Set", "Java Apache ECS" and 
 *    "Java Apache Project" must not be used to endorse or promote products 
 *    derived from this software without prior written permission.
 *
 * 5. Products derived from this software may not be called 
 *    "Java Apache Element Construction Set" nor "Java Apache ECS" appear 
 *    in their names without prior written permission of the 
 *    Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache 
 *    Project. <http://java.apache.org/>"
 *    
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 *
 */
package org.apache.ecs;

/**
    This class creates a &lt;FORM&gt; tag.

    @version $Id: Form.java,v 1.6 1999/06/08 21:29:10 stephan Exp $
    @author <a href="mailto:snagy@servletapi.com">Stephan Nagy</a>
    @author <a href="mailto:jon@clearink.com">Jon S. Stevens</a>
*/
public class Form extends MultiPartElement
{
    public static final String GET = "GET";
    public static final String get = "get";
    public static final String POST = "POST";
    public static final String post = "post";

    public static final String ENC_DEFAULT = "application/x-www-form-urlencoded";
    public static final String ENC_UPLOAD = "multipart/form-data";
    
    /**
        Private initialization routine.
    */
    {
        setElementType("form");
        setEncType ( ENC_DEFAULT );
        setAcceptCharset("UNKNOWN");
    }

    /**
        Basic constructor. You need to set the attributes using the
        set* methods.
    */
    public Form()
    {
    }

    /**
        Use the set* methods to set the values
        of the attributes.

        @param   element  set the value of &lt;FORM&gt;value&lt;/FORM&gt;
    */
    public Form(Element element)
    {
        addElement(element);
    }

    /**
        Use the set* methods to set the values
        of the attributes.

        @param   action  set the value of ACTION=""
    */
    public Form(String action)
    {
        setAction(action);
    }

    /**
        Use the set* methods to set the values
        of the attributes.

        @param   element  set the value of &lt;FORM&gt;value&lt;/FORM&gt;
        @param   action  set the value of ACTION=""
    */
    public Form(String action, Element element)
    {
        addElement(element);
        setAction(action);
    }

    /**
        Use the set* methods to set the values
        of the attributes.

        @param   action  set the value of ACTION=""
        @param   method  set the value of METHOD=""
        @param   element  set the value of &lt;FORM&gt;value&lt;/FORM&gt;
    */
    public Form(String action, String method, Element element)
    {
        addElement(element);
        setAction(action);
        setMethod(method);
    }

    /**
        Use the set* methods to set the values
        of the attributes.

        @param   action  set the value of ACTION=""
        @param   method  set the value of METHOD=""
    */
    public Form(String action, String method)
    {
        setAction(action);
        setMethod(method);
    }

    /**
        Use the set* methods to set the values
        of the attributes.

        @param   action  set the value of ACTION=""
        @param   method  set the value of METHOD=""
        @param   enctype  set the value of ENCTYPE=""
    */
    public Form(String action, String method, String enctype)
    {
        setAction(action);
        setMethod(method);
        setEncType(enctype);
    }

    /**
        Sets the ACTION="" attribute
        @param   action  the ACTION="" attribute
    */
    public Form setAction(String action)
    {
        addAttribute("action",action);
        return this;
    }

    /**
        Sets the METHOD="" attribute
        @param   method  the METHOD="" attribute
    */
    public Form setMethod(String method)
    {
        addAttribute("method",method);
        return this;
    }

    /**
        Sets the ENCTYPE="" attribute
        @param   enctype  the ENCTYPE="" attribute
    */
    public Form setEncType(String enctype)
    {
        addAttribute("enctype",enctype);
        return this;
    }

    /**
        Sets the ACCEPT="" attribute
        @param   accept  the ACCEPT="" attribute
    */
    public Form setAccept(String accept)
    {
        addAttribute("accept",accept);
        return this;
    }

    /**
        Sets the NAME="" attribute
        @param   name  the NAME="" attribute
    */
    public Form setName(String name)
    {
        addAttribute("name",name);
        return this;
    }

    /**
        Sets the ACCEPT-CHARSET="" attribute
        @param   accept  the ACCEPT-CHARSET="" attribute
    */
    public Form setAcceptCharset(String acceptcharset)
    {
        addAttribute("accept-charset",acceptcharset);
        return this;
    }

    /**
        Adds an Element to the element.
        @param  hashcode name of element for hash table
        @param  element Adds an Element to the element.
     */
    public Form addElement(String hashcode,Element element)
    {
        addElementToRegistry(hashcode,element);
        return(this);
    }

    /**
        Adds an Element to the element.
        @param  hashcode name of element for hash table
        @param  element Adds an Element to the element.
     */
    public Form addElement(String hashcode,String element)
    {
        addElementToRegistry(hashcode,element);
        return(this);
    }

    /**
        Adds an Element to the element.
        @param  element Adds an Element to the element.
     */
    public Form addElement(Element element)
    {
        addElementToRegistry(element);
        return(this);
    }

    /**
        Adds an Element to the element.
        @param  element Adds an Element to the element.
     */
    public Form addElement(String element)
    {
        addElementToRegistry(element);
        return(this);
    }
    /**
        Removes an Element from the element.
        @param hashcode the name of the element to be removed.
    */
    public Form removeElement(String hashcode)
    {
        removeElementFromRegistry(hashcode);
        return(this);
    }
}
